@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -34,10 +34,14 @@
 #include "yad.h"
 
 YadOptions options;
-GtkWidget *dialog = NULL;
+static GtkWidget *dialog = NULL;
 
 static gint ret = YAD_RESPONSE_ESC;
 
+#if GTK_CHECK_VERSION(3,0,0)
+static GtkCssProvider *css = NULL;
+#endif
+
 YadNTabs *tabs;
 gint t_sem;
 
@@ -64,17 +68,33 @@ sa_usr2 (gint sig)
 static gboolean
 keys_cb (GtkWidget *w, GdkEventKey *ev, gpointer d)
 {
-#if GTK_CHECK_VERSION(2,24,0)
-  if (ev->keyval == GDK_KEY_Escape)
-#else
-  if (ev->keyval == GDK_Escape)
-#endif
+  if (options.plug != -1)
+    return FALSE;
+
+  switch (ev->keyval)
     {
-      if (options.plug == -1 || !options.data.no_escape)
-        yad_exit (YAD_RESPONSE_ESC);
+#if GTK_CHECK_VERSION(2,24,0)
+    case GDK_KEY_Escape:
+#else
+    case GDK_Escape:
+#endif
+      if (!options.data.no_escape)
+         yad_exit (YAD_RESPONSE_ESC);
       return TRUE;
-    }
-  return FALSE;
+#if GTK_CHECK_VERSION(2,24,0)
+    case GDK_KEY_Return:
+    case GDK_KEY_KP_Enter:
+#else
+    case GDK_Return:
+    case GDK_KP_Enter:
+#endif
+      if (ev->state & GDK_CONTROL_MASK)
+        {
+          yad_exit (options.data.def_resp);
+          return TRUE;
+        }
+     }
+   return FALSE;
 }
 
 static void
@@ -191,6 +211,7 @@ create_layout (GtkWidget *dlg)
           gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
           gtk_label_set_selectable (GTK_LABEL (text), options.data.selectable_labels);
           gtk_label_set_justify (GTK_LABEL (text), options.data.text_align);
+          gtk_widget_set_state (text, GTK_STATE_NORMAL);
           switch (options.data.text_align)
             {
             case GTK_JUSTIFY_LEFT:
@@ -220,9 +241,6 @@ create_layout (GtkWidget *dlg)
     case YAD_MODE_COLOR:
       mw = color_create_widget (dlg);
       break;
-    case YAD_MODE_DND:
-      dnd_init (dlg);
-      break;
     case YAD_MODE_ENTRY:
       mw = entry_create_widget (dlg);
       break;
@@ -327,6 +345,9 @@ create_layout (GtkWidget *dlg)
       gtk_box_pack_start (GTK_BOX (layout), box, TRUE, TRUE, 0);
     }
 
+  if (options.mode == YAD_MODE_DND)
+    dnd_init (layout);
+
   return layout;
 }
 
@@ -613,7 +634,7 @@ create_dialog (void)
                   gint sh = gdk_screen_get_height (gdk_screen_get_default ());
                   options.data.posy = sh - wh + options.data.posy;
                 }
-              gtk_window_move (GTK_WINDOW (dlg), options.data.posx, options.data.posy);              
+              gtk_window_move (GTK_WINDOW (dlg), options.data.posx, options.data.posy);
             }
         }
     }
@@ -641,6 +662,11 @@ create_dialog (void)
     }
 #endif
 
+#if GTK_CHECK_VERSION(3,0,0)
+  if (css)
+    gtk_style_context_add_provider (gtk_widget_get_style_context (dlg), css, GTK_STYLE_PROVIDER_PRIORITY_USER);
+#endif
+
   return dlg;
 }
 
@@ -666,6 +692,11 @@ create_plug (void)
   if (box)
     gtk_container_add (GTK_CONTAINER (win), box);
 
+#if GTK_CHECK_VERSION(3,0,0)
+  if (css)
+    gtk_style_context_add_provider (gtk_widget_get_style_context (win), css, GTK_STYLE_PROVIDER_PRIORITY_USER);
+#endif
+
   gtk_widget_show_all (win);
 
   /* add plug data */
@@ -782,7 +813,14 @@ main (gint argc, gchar ** argv)
 
   /* parse custom gtkrc */
   if (options.gtkrc_file)
-    gtk_rc_parse (options.gtkrc_file);
+    {
+#if !GTK_CHECK_VERSION(3,0,0)
+      gtk_rc_parse (options.gtkrc_file);
+#else
+      css = gtk_css_provider_new ();
+      gtk_css_provider_load_from_path (css, options.gtkrc_file, NULL);
+#endif
+    }
 
   /* set default icons and icon theme */
   if (options.data.icon_theme)
